/*
	DynAPI Distribution
	MouseEvent Class

	The DynAPI Distribution is distributed under the terms of the GNU LGPL license.
	
	requires: dynapi.api.DynDocument
*/

function MouseEvent(type, src) {
	this.inherit('DynEvent',type,src);
	this.bubble = true;
	this._trap = false;
	this._trapSrc = null;
	this._button = 1;
	this._mouseEvent = null;
};
var p = dynapi.setPrototype('MouseEvent','DynEvent');
p.getX = function() {return this.x};
p.getY = function() {return this.y};
p.getPageX = function() {return this.pageX};
p.getPageY = function() {return this.pageY};
p.trapMouseUp = function() {return this._trap = true};
p.getButton = function() {
	if (!this._mouseEvent) return "left";
	var b = DynAPI.ua.ie? this._mouseEvent.button : this._mouseEvent.which;
	if (DynAPI.ua.ie){
		if (b==2) b=3;
		else if (b==4) b=2;
	};
	if (b==2) return "middle";
	if (b==3) return "right";
	else return "left";
};
MouseEvent._e = new MouseEvent();

MouseEvent.docEventHandler = function(e) {
	if (!e.target.lyrobj) {
		routeEvent(e);
		return true;
	}
	else {
		routeEvent(e);
		return MouseEvent._e.defaultValue;
	}
};
MouseEvent.eventHandler = function(e) {
	var dynobject = this.lyrobj;
	if (DynAPI.ua.def) {
		if (DynAPI.ua.ie) var e = window.event;
		e.cancelBubble=true;
	}
	if (DynAPI.ua.def) var realsrc = dynapi.functions.getContainerLayerOf(DynAPI.ua.ie?e.srcElement:e.target) || dynobject;
	else if (DynAPI.ua.ns4) var realsrc = e.target.lyrobj || dynobject;
	if (DynAPI.ua.ns4) {
		if (!e.target.lyrobj && (e.type=="mousedown" || e.type=="mouseup" || e.type=="click")) {
			return routeEvent(e);
		}
	}
	if (!realsrc) return true

	if (DynAPI.ua.def) {
		if (e.type=="mouseout" && realsrc.isParentOf(dynapi.functions.getContainerLayerOf(DynAPI.ua.ie?e.toElement:e.relatedTarget),true)) return true;
		if (e.type=="mouseover" && realsrc.isParentOf(dynapi.functions.getContainerLayerOf(DynAPI.ua.ie?e.fromElement:e.relatedTarget),true)) return true;
	}
	var evt = MouseEvent._e;
	var type = e.type;
	evt.type = type;
	evt.src = realsrc;
	evt.bubbleChild = null;
	evt.defaultValue = true;
	evt.bubble = true;

	evt.pageX = DynAPI.ua.ie? e.x+document.body.scrollLeft : e.pageX-window.pageXOffset;
	evt.pageY = DynAPI.ua.ie? e.y+document.body.scrollTop : e.pageY-window.pageYOffset;
	evt.x = DynAPI.ua.ie? evt.pageX-evt.src.getPageX() : e.layerX;
	evt.y = DynAPI.ua.ie? evt.pageY-evt.src.getPageY() : e.layerY;

	evt._mouseEvent = e;
	if (DynAPI.ua.def) {
		if (evt.type=='mouseover') {
			var fromL = dynapi.functions.getContainerLayerOf(DynAPI.ua.ie?e.fromElement:e.relatedTarget);
			if(fromL && fromL.isChildOf(realsrc.parent,true)) evt.preventBubble();
		}
		if (evt.type=='mouseout') {
			var toL = dynapi.functions.getContainerLayerOf(DynAPI.ua.ie?e.toElement:e.relatedTarget);
			if(toL && toL.isChildOf(realsrc.parent,true)) evt.preventBubble();
		}
	}
	else if (DynAPI.ua.ns4 && (e.type=="mouseover" || e.type=="mouseout")) evt.preventBubble();
	if (DynAPI.ua.def) {
		if (e.type=='mousedown') evt._trap = false;
		if (e.type=='mouseup' && evt._trap) evt.src = realsrc = evt._trapSrc;
	}
	evt.origin = realsrc;
	realsrc.invokeEvent(type,evt);
	if (DynAPI.ua.def) { 
		if (e.type=='mousedown') {
			if (evt._trap) evt._trapSrc = realsrc; 
		}
	}
	return evt.defaultValue;
};

DynDocument.prototype.captureMouseEvents = function() {
	if(DynAPI.ua.def&&!DynAPI.ua.ie) {
		this.doc.addEventListener("mousemove",MouseEvent.eventHandler,false);
		this.doc.addEventListener("mousedown",MouseEvent.eventHandler,false);
		this.doc.addEventListener("mouseup",MouseEvent.eventHandler,false);
		this.doc.addEventListener("mouseover",MouseEvent.eventHandler,false);
		this.doc.addEventListener("mouseout",MouseEvent.eventHandler,false);
		this.doc.addEventListener("click",MouseEvent.eventHandler,false);
		this.doc.addEventListener("dblclick",MouseEvent.eventHandler,false);
	}
	else {
		if (DynAPI.ua.ns4) this.doc.captureEvents(Event.MOUSEMOVE | Event.MOUSEDOWN | Event.MOUSEUP | Event.CLICK | Event.DBLCLICK);
		this.doc.onmousemove=this.doc.onmousedown=this.doc.onmouseup=this.doc.onclick=this.doc.onclick=this.doc.ondblclick=MouseEvent.eventHandler;
	}
};
DynDocument.prototype.releaseMouseEvents=function() {
	if(DynAPI.ua.def&&!DynAPI.ua.ie) {
		this.doc.removeEventListener("mousemove",MouseEvent.eventHandler,false);
		this.doc.removeEventListener("mousedown",MouseEvent.eventHandler,false);
		this.doc.removeEventListener("mouseup",MouseEvent.eventHandler,false);
		this.doc.removeEventListener("mouseover",MouseEvent.eventHandler,false);
		this.doc.removeEventListener("mouseout",MouseEvent.eventHandler,false);
		this.doc.removeEventListener("click",MouseEvent.eventHandler,false);
		this.doc.removeEventListener("dblclick",MouseEvent.eventHandler,false);
	}
	else {
		if (DynAPI.ua.ns4) this.doc.releaseEvents(Event.MOUSEMOVE | Event.MOUSEDOWN | Event.MOUSEUP | Event.CLICK | Event.DBLCLICK);
		this.doc.onmousemove=this.doc.onmousedown=this.doc.onmouseup=this.doc.onclick=this.doc.ondblclick=null;
	}
};

DynLayer.prototype.captureMouseEvents = function() {
	var elm=this.elm;
	if(!elm) return;
	if(DynAPI.ua.def&&!DynAPI.ua.ie) {
		elm.addEventListener("mousemove",MouseEvent.eventHandler,false);
		elm.addEventListener("mousedown",MouseEvent.eventHandler,false);
		elm.addEventListener("mouseup",MouseEvent.eventHandler,false);
		elm.addEventListener("mouseover",MouseEvent.eventHandler,false);
		elm.addEventListener("mouseout",MouseEvent.eventHandler,false);
		elm.addEventListener("click",MouseEvent.eventHandler,false);
		elm.addEventListener("dblclick",MouseEvent.eventHandler,false);
		elm.ondragstart = dynapi.functions.False;
		elm.onselectstart = dynapi.functions.False;
	}
	else {
		if (DynAPI.ua.ns4) elm.captureEvents(Event.MOUSEDOWN | Event.MOUSEUP | Event.CLICK | Event.DBLCLICK | Event.MOUSEMOVE);
		elm.onmousemove=elm.onmousedown=elm.onmouseup=elm.onmouseover=elm.onmouseout=elm.onclick=elm.ondblclick = MouseEvent.eventHandler;
	}
};
DynLayer.prototype.releaseMouseEvents=function() {
	var elm=this.elm;
	if(!elm) return;
	if(DynAPI.ua.def&&!DynAPI.ua.ie) {
		elm.removeEventListener("mousemove",MouseEvent.eventHandler,false);
		elm.removeEventListener("mousedown",MouseEvent.eventHandler,false);
		elm.removeEventListener("mouseup",MouseEvent.eventHandler,false);
		elm.removeEventListener("mouseover",MouseEvent.eventHandler,false);
		elm.removeEventListener("mouseout",MouseEvent.eventHandler,false);
		elm.removeEventListener("click",MouseEvent.eventHandler,false);
		elm.removeEventListener("dblclick",MouseEvent.eventHandler,false);
	}
	else {
		if (DynAPI.ua.ns4) elm.releaseEvents(Event.MOUSEDOWN | Event.MOUSEUP | Event.CLICK | Event.DBLCLICK | Event.MOUSEMOVE);
		elm.onmousemove=elm.onmousedown=elm.onmouseup=elm.onclick=elm.ondblclick=null;
	}
};

DynElement.prototype.assignMouseEvents = function() {
	if (this._listeners.length) this.captureMouseEvents();
};
