/*
	DynAPI Distribution
	PathAnimation Class

	The DynAPI Distribution is distributed under the terms of the GNU LGPL license.

	requires: dynapi.fx.Thread
*/

function PathAnimation(dlyr) {
	this.inherit('Thread',dlyr);
	this.paths = new Array();
	this.pathPlaying = null;
}
var p = dynapi.setPrototype('PathAnimation','Thread');

p.add = function (path, loops, resets) {
	var n = this.paths.length;
	this.paths[n] = path;
	this.setLoops(n,loops);
	this.setResets(n,resets);
	this.setFrame(n,0);
	return n;
};
p.setLoops = function (n, loops) {
	this.paths[n].loops = (loops);
};
p.setResets = function (n, resets) {
	this.paths[n].resets = (resets);
};
p.setFrame = function (n, frame) {
	this.paths[n].frame = frame;
};
p.play = function (noevt) {
	if (!this.playing) {
		this.pathPlaying = null;
		if (arguments[0]==null) arguments[0] = 0;
		if (typeof(arguments[0]) == "number") {
			this.pathPlaying = this.paths[arguments[0]];
		}
		else if (typeof(arguments[0]) == "object") {  
			this.pathPlaying = arguments[0];
			this.pathPlaying.loops = arguments[1]||false;
			this.pathPlaying.resets = arguments[2]||false;
			this.pathPlaying.frame = 0;
		}
		this.playing = true;
		if (this.dlyr!=null && noevt!=false) this.dlyr.invokeEvent("pathstart");
		this.start();
	}
};
p._Thread_stop = Thread.prototype.stop;
p.stop = function (noevt) {
	if (this.pathPlaying && this.pathPlaying.resets && !this.cancelThread && this.dlyr!=null) this.dlyr.setLocation(this.pathPlaying[0],this.pathPlaying[1]);
	this._Thread_stop();
	this.pathPlaying = null;
	this.playing = false;
	if (this.dlyr!=null && noevt!=false) this.dlyr.invokeEvent("pathcancel");
};
p.run = function () {
	if (!this.playing || this.pathPlaying==null) return;
	var anim = this.pathPlaying;
	if (anim.frame>=anim.length/2) {
		if (anim.loops) {
			anim.frame = 0;
		}
		else if (anim.resets) {
			anim.frame = 0;
			if (this.dlyr!=null) this.dlyr.setLocation(anim[0],anim[1]);
			this.stop();
			this.dlyr.invokeEvent("pathfinish");
			return;
		}
		else {
			anim.frame = 0;
			this.stop();
			this.dlyr.invokeEvent("pathfinish");
			return;
		}
	}
	if (anim.frame==0 && (this.dlyr!=null && this.dlyr.x==anim[0] && this.dlyr.y==anim[1])) {
		anim.frame += 1;
	}
	this.newX = anim[anim.frame*2];
	this.newY = anim[anim.frame*2+1];
	
	if (this.dlyr!=null) {
		this.dlyr.invokeEvent("pathrun");
		this.dlyr.setLocation(this.newX,this.newY);
	}
	anim.frame++;
};

// Path Functions

// Combines separate [x1,x2],[y1,y2] arrays into a path array [x1,y1,x2,y2]
PathAnimation.interlace = function(x,y) {
	var l = Math.max(x.length,y.length);
	var a = new Array(l*2);
	for (var i=0; i<l; i++) {
		a[i*2] = x[i];
		a[i*2+1] = y[i];
	}
	return a;
};
// Returns correct angle in radians between 2 points
PathAnimation.getNormalizedAngle = function(x1,y1,x2,y2) {
	var distx = Math.abs(x1-x2);
	var disty = Math.abs(y1-y2);
	if (distx==0 && disty==0) angle = 0;
	else if (distx==0) angle = Math.PI/2;
	else angle = Math.atan(disty/distx);
	if (x1<x2) {
		if (y1<y2) angle = Math.PI*2-angle;
	}
	else {
		if (y1<y2) angle = Math.PI+angle;
		else angle = Math.PI-angle;
	}
	return angle;
};
